// Example Code.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <Windows.h>
#include <iostream>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include "Example Code.h"
using namespace std;


const int sclkA = 4, dinA = 6, outA = 5, sclkM = 7, dinV = 0, sclkV = 1, latchV = 2, resetV = 3;//USB Pin def.
#define	MAX_PACKETS			1024
#define	DATA_PER_PACKET		62
#define	PACKET_SIZE			64
#define BAUD_RATE			62500


CTimer::CTimer()
{
    if (!::QueryPerformanceFrequency(&m_llFrequency))
        m_llFrequency.QuadPart = 0;
	m_uiResolution = 1;
    while (::timeBeginPeriod(m_uiResolution) == TIMERR_NOCANDO)
	{
		if (++m_uiResolution == 20)
			break;
	}
}

CTimer::~CTimer()
{
    ::timeEndPeriod(m_uiResolution);
}

double CTimer::Seconds() const
{
    double dTime;
    if (m_llFrequency.QuadPart != 0)
    {
        LARGE_INTEGER llTime;
        ::QueryPerformanceCounter(&llTime);     // get current time
        dTime = (double)(llTime.QuadPart);
        dTime /= m_llFrequency.QuadPart;
    }
    else
        dTime = ::timeGetTime() / 1000.0;
    return dTime;
}



//Opens device by name or serial number. Returns device handle in ftHandle. Returns FT error types. Returns FT error if already open.
FT_STATUS openByDesSer(string DeviceDescription, string DeviceSerialNumber, FT_HANDLE &ftHandle)
{
	FT_STATUS ftStatus; 
	FT_HANDLE ftHandleTemp;
	DWORD numDevs; 
	DWORD Flags; 
	DWORD ID; 
	DWORD Type; 
	DWORD LocId; 
	char SerialNumber[16]; 
	char Description[64];
	string desc;
	string ser;

	ftStatus = FT_CreateDeviceInfoList(&numDevs); 
	if (!(ftStatus == FT_OK)) 
	{ 
		return FT_IO_ERROR;
	}

	if (numDevs > 0) 
	{ 
		// get information for device 
		for (int i = 0; i < (int) numDevs; i++) 
		{ 
			ftStatus = FT_GetDeviceInfoDetail(i, &Flags, &Type, &ID, &LocId, SerialNumber, Description, &ftHandleTemp);
			if (ftStatus == FT_OK) 
			{ 
				desc= Description;
				ser= SerialNumber;
				if (ser.compare(DeviceSerialNumber)==0||desc.compare(DeviceDescription)==0)
				{
					ftStatus= FT_Open(i, &ftHandle);
					if (ftStatus==FT_OK)
						return FT_OK;
				}
			} 
		} 
	} 
	else
	{
		return FT_DEVICE_NOT_FOUND;
	}
	return FT_OTHER_ERROR;
}

//Closes a device
FT_STATUS closeDevice(FT_HANDLE ftHandle)
{
	return FT_Close(ftHandle);
}



int main(int argc, char* argv[])//_tmain(int argc, _TCHAR* argv[])
{
	string path= "Alder Board";//argv[0];
	//path= path.substr(path.find_last_of("\\")+1, path.find(".exe")-path.find_last_of("\\")-1);

	CTimer cTimer;

	FT_STATUS ftStatus;
	FT_HANDLE ftHandle; 
	//DWORD BytesWritten;
	unsigned char Mask = (1<<sclkA)|(1<<dinA)|(1<<outA)|(1<<sclkM)|(0<<dinV)|(0<<sclkV)|(0<<latchV)|(1<<resetV);
	unsigned char Mode = 0x4;//sync mode
	unsigned char tx[MAX_PACKETS*DATA_PER_PACKET*3];
	memset(tx, 0, sizeof(tx));
	unsigned char rx[MAX_PACKETS*DATA_PER_PACKET*3];

	FILE* plog;
	fopen_s(&plog, "FT232 Log.txt", "a");

	DWORD BytesWritten= 0;
	DWORD RxBytes= 0;


	double dStart, dEnd, dMid;
	double dWrite[20];
	double dRead[20];
	double dTotal[20];

	if (openByDesSer(path, path, ftHandle)==FT_OK){
		ftStatus = FT_ResetDevice(ftHandle);
		cout<<"Reset FTcode: "<<ftStatus<<endl;
		ftStatus= FT_SetBitMode(ftHandle, Mask, Mode);
		cout<<"SetBitMode FTcode: "<<ftStatus<<endl;
		ftStatus= FT_SetBaudRate(ftHandle, BAUD_RATE);
		fprintf(plog, "Baud rate: %d, Byte rate: %d\n", BAUD_RATE, BAUD_RATE*16);
		cout<<"SetBaud FTcode: "<<ftStatus<<endl;
		ftStatus = FT_SetLatencyTimer(ftHandle, 250);	// So that delays would be obvious
		cout<<"Latency Timer FTcode: "<<ftStatus<<endl;

		for (int tr= 1;tr<=MAX_PACKETS;++tr)	// For each transfer size from 1*64 to MAX_PACKETS*64 (64Kb)
		{
			FT_SetUSBParameters(ftHandle, tr*PACKET_SIZE, 0); 
			Sleep(50);
			// Write in multiples of the transfer size, max 50 multiples or 3 multiples for trasnfer size of MAX_PACKETS*64
			int mult= 1;
			for (;mult<2 && mult*tr<=3*MAX_PACKETS;++mult)
			{
				dStart= cTimer.Seconds();
				// Replicate 50 times
				
				for (int i= 0;i<25;++i)
				{

					dStart= cTimer.Seconds();
					FT_Write(ftHandle, tx, mult*tr*DATA_PER_PACKET, &BytesWritten);
					dMid= cTimer.Seconds();
					FT_Read(ftHandle,rx,mult*tr*DATA_PER_PACKET,&RxBytes);
					dEnd= cTimer.Seconds();
					dWrite[i]= dMid-dStart;
					dRead[i]= dEnd-dMid;
					dTotal[i]= dEnd-dStart;
					
				}
				fprintf(plog, "Write,%d,%d,", tr, mult);
				for (int j= 0;j<20;++j)
				{
					fprintf(plog, "%g,", dWrite[j]);
				}
				fprintf(plog,"\n");
				fprintf(plog, "Read,%d,%d,", tr, mult);
				for (int j= 0;j<20;++j)
				{
					fprintf(plog, "%g,", dRead[j]);
				}
				fprintf(plog,"\n");
				fprintf(plog, "Total,%d,%d,", tr, mult);
				cout<<"T-"<<tr<<","<<mult<<",";
				for (int j= 0;j<20;++j)
				{
					fprintf(plog, "%g,", dTotal[j]);
				}
				fprintf(plog,"\n");


			}

			fflush(plog);
			Sleep(50);
		}


		closeDevice(&ftHandle);
	}
	fclose(plog);
	//system("pause");
	return 0;
}

